@@ -29,6 +29,7 @@ pub struct AardvarkEntry<'a> {
     pub container_ips_v6: Vec<Ipv6Addr>,
     pub container_names: Vec<String>,
     pub container_dns_servers: &'a Option<Vec<IpAddr>>,
+    pub is_internal: bool,
 }
 
 #[derive(Debug, Clone)]
@@ -115,21 +116,52 @@ impl Aardvark {
         // After https://github.com/containers/aardvark-dns/pull/148 this command
         // will block till aardvark-dns's parent process returns back and let
         // aardvark inherit all the fds.
-        Command::new(aardvark_args[0])
+        let out = Command::new(aardvark_args[0])
             .args(&aardvark_args[1..])
-            .stdin(Stdio::inherit())
-            .stdout(Stdio::inherit())
-            .stderr(Stdio::inherit())
+            .stdin(Stdio::null())
+            .stdout(Stdio::null())
             // set RUST_LOG for aardvark
             .env("RUST_LOG", log::max_level().as_str())
             .output()?;
 
-        Ok(())
+        if out.status.success() {
+            return Ok(());
+        }
+        if out.stderr.is_empty() {
+            return Err(std::io::Error::new(
+                std::io::ErrorKind::Other,
+                "aardvark-dns exited unexpectedly without error message",
+            ));
+        }
+        // aardvark-dns failed capture stderr
+        let msg = String::from_utf8(out.stderr).map_err(|e| {
+            std::io::Error::new(
+                std::io::ErrorKind::Other,
+                format!("failed to parse aardvark-dns stderr message: {e}"),
+            )
+        })?;
+
+        Err(std::io::Error::new(
+            std::io::ErrorKind::Other,
+            format!("aardvark-dns failed to start: {}", msg.trim()),
+        ))
     }
 
-    fn check_netns(&self, pid: pid_t) -> Result<()> {
-        let cur_ns = fs::read_link("/proc/self/ns/net")?;
-        let aardvark_ns = fs::read_link(format!("/proc/{pid}/ns/net"))?;
+    fn check_netns(&self, pid: pid_t) {
+        // This should never fail but ignore errors anyway
+        let cur_ns = match fs::read_link("/proc/self/ns/net") {
+            Ok(p) => p,
+            Err(_) => return,
+        };
+        // This might fail
+        let aardvark_ns = match fs::read_link(format!("/proc/{pid}/ns/net")) {
+            Ok(p) => p,
+            // In case of errors ignore them and do not warn. When the process is exiting then
+            // several different errors can happen. I have observed ENOENT, ESRCH and EACCES so
+            // to be safe just ignore all errors as this warning here is just best effort anyway.
+            // https://github.com/containers/podman/issues/22103
+            Err(_) => return,
+        };
 
         if aardvark_ns != cur_ns {
             // netns does not match, this means dns will not work.
@@ -142,8 +174,6 @@ impl Aardvark {
                 self.config.display()
             );
         }
-
-        Ok(())
     }
 
     pub fn notify(&self, start: bool, is_update: bool) -> NetavarkResult<()> {
@@ -151,21 +181,14 @@ impl Aardvark {
             Ok(pid) => {
                 match signal::kill(Pid::from_raw(pid), Signal::SIGHUP) {
                     Ok(_) => {
+                        // We do not want to check the netns when doing an update
+                        // this is not working because podman does not enter the
+                        // rootless netns for the update as we only change the file
+                        // and send SIGHUP.
                         if !is_update {
-                            match self.check_netns(pid) {
-                                Ok(_) => return Ok(()),
-                                Err(e) => {
-                                    // If the error is ENOENT it means the process must have died in
-                                    // the meantime so drop down below to start a new server process.
-                                    if e.kind() != std::io::ErrorKind::NotFound {
-                                        return Err(NetavarkError::wrap(
-                                            "check aardvark-dns netns",
-                                            e.into(),
-                                        ));
-                                    }
-                                }
-                            }
+                            self.check_netns(pid)
                         }
+                        return Ok(());
                     }
                     Err(err) => {
                         // ESRCH == process does not exists
@@ -188,7 +211,7 @@ impl Aardvark {
         Ok(())
     }
 
-    pub fn commit_entries(&self, entries: Vec<AardvarkEntry>) -> Result<()> {
+    pub fn commit_entries(&self, entries: &[AardvarkEntry]) -> Result<()> {
         // Acquire fs lock to ensure other instance of aardvark cannot commit
         // or start aardvark instance till already running instance has not
         // completed its `commit` phase.
@@ -199,6 +222,7 @@ impl Aardvark {
             .read(true)
             .write(true)
             .create(true)
+            .truncate(true)
             .open(&lockfile_path)
         {
             Ok(file) => file,
@@ -216,8 +240,13 @@ impl Aardvark {
             ));
         }
 
-        for entry in &entries {
-            let path = Path::new(&self.config).join(entry.network_name);
+        for entry in entries {
+            let mut path = Path::new(&self.config).join(entry.network_name);
+            if entry.is_internal {
+                let new_path = Path::new(&self.config).join(entry.network_name.to_owned() + "%int");
+                let _ = std::fs::rename(&path, &new_path);
+                path = new_path;
+            }
 
             let file = match OpenOptions::new().write(true).create_new(true).open(&path) {
                 Ok(mut f) => {
@@ -315,14 +344,28 @@ impl Aardvark {
 
     pub fn commit_netavark_entries(&self, entries: Vec<AardvarkEntry>) -> NetavarkResult<()> {
         if !entries.is_empty() {
-            self.commit_entries(entries)?;
-            self.notify(true, false)?;
+            self.commit_entries(&entries)?;
+            match self.notify(true, false) {
+                Ok(_) => (),
+                Err(e) => {
+                    if let Err(err) = self.delete_from_netavark_entries(&entries) {
+                        log::warn!(
+                            "Failed to delete aardvark-dns entries after failed start: {err}"
+                        );
+                    };
+                    return Err(e);
+                }
+            };
         }
         Ok(())
     }
 
     pub fn delete_entry(&self, container_id: &str, network_name: &str) -> Result<()> {
-        let path = Path::new(&self.config).join(network_name);
+        let mut path = Path::new(&self.config).join(network_name);
+        if !path.exists() {
+            path = Path::new(&self.config).join(network_name.to_owned() + "%int");
+        }
+
         let file_content = fs::read_to_string(&path)?;
         let lines: Vec<&str> = file_content.split_terminator('\n').collect();
 
@@ -352,7 +395,7 @@ impl Aardvark {
     pub fn modify_network_dns_servers(
         &self,
         network_name: &str,
-        network_dns_servers: &Vec<String>,
+        network_dns_servers: &[String],
     ) -> NetavarkResult<()> {
         let mut dns_servers_modified = false;
         let path = Path::new(&self.config).join(network_name);
@@ -417,8 +460,8 @@ impl Aardvark {
         Ok(())
     }
 
-    pub fn delete_from_netavark_entries(&self, entries: Vec<AardvarkEntry>) -> NetavarkResult<()> {
-        for entry in &entries {
+    pub fn delete_from_netavark_entries(&self, entries: &[AardvarkEntry]) -> NetavarkResult<()> {
+        for entry in entries {
             self.delete_entry(entry.container_id, entry.network_name)?;
         }
         self.notify(false, false)
