@@ -3,7 +3,7 @@ use crate::firewall::varktables::helpers::{
     add_chain_unique, append_unique, remove_if_rule_exists,
 };
 use crate::firewall::varktables::types::TeardownPolicy::{Never, OnComplete};
-use crate::network::internal_types::PortForwardConfig;
+use crate::network::internal_types::{IsolateOption, PortForwardConfig};
 use ipnet::IpNet;
 use iptables::IPTables;
 use log::debug;
@@ -15,7 +15,10 @@ const FILTER: &str = "filter";
 const POSTROUTING: &str = "POSTROUTING";
 const PREROUTING: &str = "PREROUTING";
 const NETAVARK_FORWARD: &str = "NETAVARK_FORWARD";
+const NETAVARK_FIREWALL_RULE_BUILDER: &str = "-m comment --comment 'netavark firewall rules' -j ";
+const NETAVARK_INPUT: &str = "NETAVARK_INPUT";
 const OUTPUT: &str = "OUTPUT";
+const INPUT: &str = "INPUT";
 const FORWARD: &str = "FORWARD";
 const ACCEPT: &str = "ACCEPT";
 const NETAVARK_HOSTPORT_DNAT: &str = "NETAVARK-HOSTPORT-DNAT";
@@ -26,6 +29,7 @@ const MARK: &str = "MARK";
 const DNAT: &str = "DNAT";
 const NETAVARK_ISOLATION_1: &str = "NETAVARK_ISOLATION_1";
 const NETAVARK_ISOLATION_2: &str = "NETAVARK_ISOLATION_2";
+const NETAVARK_ISOLATION_3: &str = "NETAVARK_ISOLATION_3";
 
 const CONTAINER_DN_CHAIN: &str = "NETAVARK-DN-";
 
@@ -194,7 +198,8 @@ pub fn get_network_chains<'a>(
     network_hash_name: &'a str,
     is_ipv6: bool,
     interface_name: String,
-    isolation: bool,
+    isolation: IsolateOption,
+    dns_port: u16,
 ) -> Vec<VarkChain<'a>> {
     let mut chains = Vec::new();
     let prefixed_network_hash_name = format!("{}-{}", "NETAVARK", network_hash_name);
@@ -209,7 +214,7 @@ pub fn get_network_chains<'a>(
     hashed_network_chain.create = true;
 
     hashed_network_chain.build_rule(VarkRule::new(
-        format!("-d {} -j {}", network, ACCEPT),
+        format!("-d {network} -j {ACCEPT}"),
         Some(TeardownPolicy::OnComplete),
     ));
 
@@ -218,7 +223,7 @@ pub fn get_network_chains<'a>(
         multicast_dest = MULTICAST_NET_V6;
     }
     hashed_network_chain.build_rule(VarkRule::new(
-        format!("! -d {} -j {}", multicast_dest, MASQUERADE),
+        format!("! -d {multicast_dest} -j {MASQUERADE}"),
         Some(TeardownPolicy::OnComplete),
     ));
     chains.push(hashed_network_chain);
@@ -227,18 +232,47 @@ pub fn get_network_chains<'a>(
     let mut postrouting_chain =
         VarkChain::new(conn, NAT.to_string(), POSTROUTING.to_string(), None);
     postrouting_chain.build_rule(VarkRule::new(
-        format!("-s {} -j {}", network, prefixed_network_hash_name),
+        format!("-s {network} -j {prefixed_network_hash_name}"),
         Some(TeardownPolicy::OnComplete),
     ));
     chains.push(postrouting_chain);
 
     // FORWARD chain
-    let mut forward_chain = VarkChain::new(conn, FILTER.to_string(), FORWARD.to_string(), None);
+    let mut forward_chain: VarkChain<'_> =
+        VarkChain::new(conn, FILTER.to_string(), FORWARD.to_string(), None);
+
+    // INPUT chain
+    let mut input_chain: VarkChain<'_> =
+        VarkChain::new(conn, FILTER.to_string(), INPUT.to_string(), None);
 
     // used to prepend specific rules
     let mut ind = 1;
 
-    if isolation {
+    // NETAVARK_ISOLATION_2
+    // NETAVARK_ISOLATION_2 chain must always exist,
+    // because non-isolation creates DROP rule in NETAVARK_ISOLATION_3
+    // and NETAVARK_ISOLATION_3 references this as a jump target.
+    let mut netavark_isolation_chain_2 = VarkChain::new(
+        conn,
+        FILTER.to_string(),
+        NETAVARK_ISOLATION_2.to_string(),
+        None,
+    );
+    netavark_isolation_chain_2.create = true;
+
+    // NETAVARK_ISOLATION_3
+    // NETAVARK_ISOLATION_3 chain must exist when IsolateOption is Never or Strict.
+    // bacause non-isolation creates DROP rule in NETAVARK_ISOLATION_3.
+    // and strict isolation references NETAVARK_ISOLATION_3 as a jump target.
+    let mut netavark_isolation_chain_3 = VarkChain::new(
+        conn,
+        FILTER.to_string(),
+        NETAVARK_ISOLATION_3.to_string(),
+        None,
+    );
+    netavark_isolation_chain_3.create = true;
+
+    if let IsolateOption::Normal | IsolateOption::Strict = isolation {
         debug!("Add extra isolate rules");
         // NETAVARK_ISOLATION_1
         let mut netavark_isolation_chain_1 = VarkChain::new(
@@ -249,51 +283,71 @@ pub fn get_network_chains<'a>(
         );
         netavark_isolation_chain_1.create = true;
 
-        // NETAVARK_ISOLATION_2
-        let mut netavark_isolation_chain_2 = VarkChain::new(
-            conn,
-            FILTER.to_string(),
-            NETAVARK_ISOLATION_2.to_string(),
-            None,
-        );
-        netavark_isolation_chain_2.create = true;
-
         // -A FORWARD -j NETAVARK_ISOLATION_1
         forward_chain.build_rule(VarkRule {
-            rule: format!("-j {}", NETAVARK_ISOLATION_1),
+            rule: format!("-j {NETAVARK_ISOLATION_1}"),
             position: Some(ind),
             td_policy: Some(TeardownPolicy::OnComplete),
         });
 
-        // NETAVARK_ISOLATION_1 -i bridge_name ! -o bridge_name -j DROP
+        let netavark_isolation_1_target = if let IsolateOption::Strict = isolation {
+            // NETAVARK_ISOLATION_1 -i bridge_name ! -o bridge_name -j NETAVARK_ISOLATION_3
+            NETAVARK_ISOLATION_3
+        } else {
+            // NETAVARK_ISOLATION_1 -i bridge_name ! -o bridge_name -j NETAVARK_ISOLATION_2
+            NETAVARK_ISOLATION_2
+        };
         netavark_isolation_chain_1.build_rule(VarkRule {
             rule: format!(
-                "-i {} ! -o {} -j {}",
-                interface_name, interface_name, NETAVARK_ISOLATION_2
+                "-i {interface_name} ! -o {interface_name} -j {netavark_isolation_1_target}"
             ),
             position: Some(ind),
             td_policy: Some(TeardownPolicy::OnComplete),
         });
 
-        // NETAVARK_ISOLATION_2 -i bridge_name ! -o bridge_name -j DROP
+        // NETAVARK_ISOLATION_2 -o bridge_name -j DROP
         netavark_isolation_chain_2.build_rule(VarkRule {
             rule: format!("-o {} -j {}", interface_name, "DROP"),
             position: Some(ind),
             td_policy: Some(TeardownPolicy::OnComplete),
         });
 
+        // NETAVARK_ISOLATION_3 -j NETAVARK_ISOLATION_2
+        netavark_isolation_chain_3.build_rule(VarkRule {
+            rule: format!("-j {NETAVARK_ISOLATION_2}"),
+            position: Some(ind),
+            td_policy: Some(TeardownPolicy::Never),
+        });
+
         ind += 1;
 
         // PUSH CHAIN
         chains.push(netavark_isolation_chain_1);
-        chains.push(netavark_isolation_chain_2)
+    } else {
+        // create DROP rule for non-isolations to enforce strict isolation rules.
+
+        // NETAVARK_ISOLATION_3 -o bridge_name -j DROP
+        netavark_isolation_chain_3.build_rule(VarkRule {
+            rule: format!("-o {} -j {}", interface_name, "DROP"),
+            position: Some(ind),
+            td_policy: Some(TeardownPolicy::OnComplete),
+        });
+
+        // NETAVARK_ISOLATION_3 -j NETAVARK_ISOLATION_2
+        netavark_isolation_chain_3.build_rule(VarkRule {
+            rule: format!("-j {NETAVARK_ISOLATION_2}"),
+            // position +1 to place this rule under all of NETAVARK_ISOLATION_3 DROP rules.
+            position: Some(ind + 1),
+            td_policy: Some(TeardownPolicy::Never),
+        });
     }
 
+    // PUSH CHAIN
+    chains.push(netavark_isolation_chain_2);
+    chains.push(netavark_isolation_chain_3);
+
     forward_chain.build_rule(VarkRule {
-        rule: format!(
-            "-m comment --comment 'netavark firewall plugin rules' -j {}",
-            NETAVARK_FORWARD
-        ),
+        rule: format!("{} {}", NETAVARK_FIREWALL_RULE_BUILDER, NETAVARK_FORWARD),
         position: Some(ind),
         td_policy: Some(TeardownPolicy::Never),
     });
@@ -304,20 +358,51 @@ pub fn get_network_chains<'a>(
         VarkChain::new(conn, FILTER.to_string(), NETAVARK_FORWARD.to_string(), None);
     netavark_forward_chain.create = true;
 
+    // Add NETAVARK_INPUT chain to INPUT chain
+    input_chain.build_rule(VarkRule {
+        rule: format!("{} {}", NETAVARK_FIREWALL_RULE_BUILDER, NETAVARK_INPUT),
+        position: Some(1),
+        td_policy: Some(TeardownPolicy::Never),
+    });
+    chains.push(input_chain);
+
+    // NETAVARK_INPUT
+    let mut netavark_input_chain =
+        VarkChain::new(conn, FILTER.to_string(), NETAVARK_INPUT.to_string(), None);
+    netavark_input_chain.create = true;
+
+    // Always add ACCEPT rules in firewall for dns traffic from containers
+    // to gateway when using bridge network with internal dns.
+    for proto in ["udp", "tcp"] {
+        netavark_input_chain.build_rule(VarkRule::new(
+            format!(
+                "-p {} -s {} --dport {} -j {}",
+                proto, network, dns_port, ACCEPT
+            ),
+            Some(TeardownPolicy::OnComplete),
+        ));
+    }
+    chains.push(netavark_input_chain);
+
+    // Drop all invalid packages, due a race the container source ip could be leaked on the local
+    // network and we should avoid that, https://bugzilla.redhat.com/show_bug.cgi?id=2230144
+    // This should't harm anything so just add one global rule instead of filtering per subnet.
+    netavark_forward_chain.build_rule(VarkRule::new(
+        "-m conntrack --ctstate INVALID -j DROP".to_string(),
+        Some(TeardownPolicy::Never),
+    ));
+
     // Create incoming traffic rule
     // CNI did this by IP address, this is implemented per subnet
     netavark_forward_chain.build_rule(VarkRule::new(
-        format!(
-            "-d {} -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT",
-            network
-        ),
+        format!("-d {network} -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT"),
         Some(TeardownPolicy::OnComplete),
     ));
 
     // Create outgoing traffic rule
     // CNI did this by IP address, this is implemented per subnet
     netavark_forward_chain.build_rule(VarkRule::new(
-        format!("-s {} -j ACCEPT", network),
+        format!("-s {network} -j ACCEPT"),
         Some(TeardownPolicy::OnComplete),
     ));
     chains.push(netavark_forward_chain);
@@ -370,14 +455,14 @@ pub fn get_port_forwarding_chains<'a>(
     // PREROUTING
     let mut prerouting_chain = VarkChain::new(conn, NAT.to_string(), PREROUTING.to_string(), None);
     prerouting_chain.build_rule(VarkRule::new(
-        format!("-j {} -m addrtype --dst-type LOCAL", NETAVARK_HOSTPORT_DNAT),
+        format!("-j {NETAVARK_HOSTPORT_DNAT} -m addrtype --dst-type LOCAL"),
         Some(TeardownPolicy::Never),
     ));
 
     //  OUTPUT
     let mut output_chain = VarkChain::new(conn, NAT.to_string(), OUTPUT.to_string(), None);
     output_chain.build_rule(VarkRule::new(
-        format!("-j {} -m addrtype --dst-type LOCAL", NETAVARK_HOSTPORT_DNAT),
+        format!("-j {NETAVARK_HOSTPORT_DNAT} -m addrtype --dst-type LOCAL"),
         Some(TeardownPolicy::Never),
     ));
 
@@ -390,7 +475,7 @@ pub fn get_port_forwarding_chains<'a>(
     );
     netavark_hostport_setmark.create = true;
     netavark_hostport_setmark.build_rule(VarkRule::new(
-        format!("-j {}  --set-xmark {}/{}", MARK, HEXMARK, HEXMARK),
+        format!("-j {MARK}  --set-xmark {HEXMARK}/{HEXMARK}"),
         Some(TeardownPolicy::Never),
     ));
     chains.push(netavark_hostport_setmark);
@@ -405,8 +490,7 @@ pub fn get_port_forwarding_chains<'a>(
     netavark_hostport_masq_chain.create = true;
     netavark_hostport_masq_chain.build_rule(VarkRule::new(
         format!(
-            "-j {} -m comment --comment 'netavark portfw masq mark' -m mark --mark {}/{}",
-            MASQUERADE, HEXMARK, HEXMARK
+            "-j {MASQUERADE} -m comment --comment 'netavark portfw masq mark' -m mark --mark {HEXMARK}/{HEXMARK}"
         ),
         Some(TeardownPolicy::Never),
     ));
@@ -417,7 +501,7 @@ pub fn get_port_forwarding_chains<'a>(
     let mut postrouting = VarkChain::new(conn, NAT.to_string(), POSTROUTING.to_string(), None);
     // This rule must be in the first position
     postrouting.build_rule(VarkRule {
-        rule: format!("-j {} ", NETAVARK_HOSTPORT_MASK),
+        rule: format!("-j {NETAVARK_HOSTPORT_MASK} "),
         position: Some(1),
         td_policy: Some(Never),
     });
@@ -437,16 +521,18 @@ pub fn get_port_forwarding_chains<'a>(
             }
             let mut ip_value = dns_ip.to_string();
             if is_ipv6 {
-                ip_value = format!("[{}]", ip_value)
+                ip_value = format!("[{ip_value}]")
             }
             netavark_hostport_dn_chain.create = true;
-            netavark_hostport_dn_chain.build_rule(VarkRule::new(
-                format!(
-                    "-j {} -d {} -p {} --dport {} --to-destination {}:{}",
-                    DNAT, dns_ip, "udp", 53, ip_value, pfwd.dns_port
-                ),
-                Some(TeardownPolicy::OnComplete),
-            ));
+            for proto in ["udp", "tcp"] {
+                netavark_hostport_dn_chain.build_rule(VarkRule::new(
+                    format!(
+                        "-j {} -d {} -p {} --dport {} --to-destination {}:{}",
+                        DNAT, dns_ip, proto, 53, ip_value, pfwd.dns_port
+                    ),
+                    Some(TeardownPolicy::OnComplete),
+                ));
+            }
         }
     }
 
@@ -518,9 +604,8 @@ pub fn get_port_forwarding_chains<'a>(
                 // if a destination ip address is provided, we need to alter
                 // the rule a bit
                 if let Some(host_ip) = host_ip {
-                    dn_setmark_rule_localhost =
-                        format!("{} -d {}", dn_setmark_rule_localhost, host_ip);
-                    dn_setmark_rule_subnet = format!("{} -d {}", dn_setmark_rule_subnet, host_ip);
+                    dn_setmark_rule_localhost = format!("{dn_setmark_rule_localhost} -d {host_ip}");
+                    dn_setmark_rule_subnet = format!("{dn_setmark_rule_subnet} -d {host_ip}");
                 }
 
                 // dn container (the actual port usages)
@@ -530,7 +615,7 @@ pub fn get_port_forwarding_chains<'a>(
 
                 let mut container_ip_value = container_ip.to_string();
                 if is_ipv6 {
-                    container_ip_value = format!("[{}]", container_ip_value)
+                    container_ip_value = format!("[{container_ip_value}]")
                 }
                 let mut container_port = i.container_port.to_string();
                 if is_range {
@@ -549,7 +634,7 @@ pub fn get_port_forwarding_chains<'a>(
                 // if a destination ip address is provided, we need to alter
                 // the rule a bit
                 if let Some(host_ip) = host_ip {
-                    dnat_rule = format!("{} -d {}", dnat_rule, host_ip)
+                    dnat_rule = format!("{dnat_rule} -d {host_ip}")
                 }
                 netavark_hashed_dn_chain.build_rule(VarkRule::new(dnat_rule, None));
             }
